@@ -21,14 +21,8 @@ from pathlib import Path
 import shutil
 from tempfile import NamedTemporaryFile
 
-from typing import (
-    Any,
-    Iterator,
-    List,
-    Optional,
-    Sequence,
-    Union,
-)
+from typing import Any, Iterator, List, Optional, Sequence, Union
+import _csv
 
 from .point import Point
 
@@ -88,10 +82,15 @@ class Storage(ABC):  # pragma: no cover
         return True
 
     @abstractmethod
-    def __iter__(self) -> Iterator:
+    def __iter__(self) -> Iterator[Union[MemStorageItem, CSVStorageItem]]:
         """Return a generator for items in storage."""
         ...
 
+    @abstractmethod
+    def __len__(self) -> int:
+        """Return the number of items."""
+        ...
+
     @abstractmethod
     def append(self, points: List[Any], temporary: bool = False) -> None:
         """Append points to the store.
@@ -189,12 +188,12 @@ class CSVStorage(Storage):
         access_mode: str = "r+",
         flush_on_insert: bool = True,
         newline: Optional[str] = "",
-        **kwargs,
+        **kwargs: Any,
     ) -> None:
         """Init a CSVStorage instance.
 
         This will init a file object to the specified filepath. No reads are
-        performed by default, so we don't know if the data is sorted and
+        performed by default, so we don't know if data is present and
         therefore, the _initially_empty attribute is set to False.
 
         Args:
@@ -202,6 +201,7 @@ class CSVStorage(Storage):
             create_dirs: Create parent subdirectories.
             encoding: File encoding.
             access_mode: File access mode.
+            flush_on_insert: Whether or not to flush IO buffer immediately.
             newline: Determines how to parse newline characters from the stream
         """
         super().__init__()
@@ -259,7 +259,7 @@ class CSVStorage(Storage):
 
         return True
 
-    def __iter__(self) -> Iterator:
+    def __iter__(self) -> _csv.reader:  # type: ignore
         """Return a CSV reader object that can be iterated over."""
         self._handle.seek(0)
 
@@ -271,7 +271,9 @@ class CSVStorage(Storage):
 
         return sum(1 for _ in self._handle)
 
-    def append(self, items: List[CSVStorageItem], temporary=False) -> None:
+    def append(
+        self, items: List[CSVStorageItem], temporary: bool = False
+    ) -> None:
         """Append points to the CSV store.
 
         Args:
@@ -435,14 +437,18 @@ class MemoryStorage(Storage):
     Memory is cleaned up along with the parent process.
 
     Attributes:
-        index_intact: Data is stored according to the index sorter.
+        _initially_empty: No data in the storage instance.
+        _memory: List of Points.
+        _temp_memory: List of Points.
 
     Usage:
         >>> from tinyflux import MemoryStorage
         >>> db = TinyFlux(storage=MemoryStorage)
     """
 
+    _initially_empty: bool
     _memory: List[MemStorageItem]
+    _temp_memory: List[MemStorageItem]
 
     def __init__(self) -> None:
         """Init a MemoryStorage instance."""
@@ -451,7 +457,7 @@ class MemoryStorage(Storage):
         self._memory = []
         self._temp_memory: List[MemStorageItem] = []
 
-    def __iter__(self) -> Iterator:
+    def __iter__(self) -> Iterator[Point]:
         """Return a generator to memory that can be iterated over."""
         for point in self._memory:
             yield point
@@ -460,7 +466,9 @@ class MemoryStorage(Storage):
         """Return the number of items."""
         return len(self._memory)
 
-    def append(self, items: List[MemStorageItem], temporary=False) -> None:
+    def append(
+        self, items: List[MemStorageItem], temporary: bool = False
+    ) -> None:
         """Append points to the memory.
 
         Args:
@@ -509,6 +517,9 @@ class MemoryStorage(Storage):
 
     def _deserialize_timestamp(self, item: MemStorageItem) -> datetime:
         """Deserialize timestamp from a point."""
+        if not item.time:  # pragma: no cover
+            raise ValueError
+
         return item.time
 
     def _init_temp_storage(self) -> None:
